@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/client.c,v 11.1 2001/09/03 22:02:10 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/client.c,v 11.3 2003/03/23 18:02:01 jcb Rel $
  * client.c
  * Provides generic client support. It connects to the controller,
  * and maintains a game data structure in response to the Controller
@@ -17,7 +17,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/client.c,v 11.1 2001/09/03 22:02:10 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/client.c,v 11.3 2003/03/23 18:02:01 jcb Rel $";
 
 #include "sysdep.h"
 #include <assert.h>
@@ -29,10 +29,14 @@ Game *client_init(char *address) {
   int i;
   Game *g;
 
-  fd = connect_to_host(address);
-  if ( fd == INVALID_SOCKET ) {
-    perror("client_init: connect_to_host failed");
-    return 0;
+  if ( strcmp("-",address) == 0 ) {
+    fd = STDOUT_FILENO;
+  } else {
+    fd = connect_to_host(address);
+    if ( fd == INVALID_SOCKET ) {
+      perror("client_init: connect_to_host failed");
+      return 0;
+    }
   }
 
   /* alloc the structures */
@@ -163,6 +167,8 @@ TileSet *client_find_sets(PlayerP p, Tile d, int mj, PlayerP *pcopies,
     }
   } else {
     /* no discard. Look for closed sets */
+    /* Because we should return pungs, then chows, then pairs,
+       we have to go through the tiles three times, which is tedious. */
     for (j=0; j < MaxTile; j++) seen[j] = 0;
     for (j=0; j < p->num_concealed; j++) {
       t = p->concealed[j];
@@ -173,12 +179,22 @@ TileSet *client_find_sets(PlayerP p, Tile d, int mj, PlayerP *pcopies,
 	tsp->tile = t;
 	tsp++; cp++;
       }
+    }    
+    for (j=0; j < MaxTile; j++) seen[j] = 0;
+    for (j=0; j < p->num_concealed; j++) {
+      t = p->concealed[j];
+      if ( seen[t]++ ) continue; /* don't enter sets twice */
       copy_player(cp,p);
       if ( player_forms_closed_chow(cp,t,Lower) && (!mj || player_can_mah_jong(cp,HiddenTile,flags))) {
 	tsp->type = ClosedChow;
 	tsp->tile = t;
 	tsp++; cp++;
       }
+    }    
+    for (j=0; j < MaxTile; j++) seen[j] = 0;
+    for (j=0; j < p->num_concealed; j++) {
+      t = p->concealed[j];
+      if ( seen[t]++ ) continue; /* don't enter sets twice */
       copy_player(cp,p);
       if ( player_forms_closed_pair(cp,t) && (!mj || player_can_mah_jong(cp,HiddenTile,flags))) {
 	tsp->type = ClosedPair;
