@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/game.c,v 11.8 2002/03/30 22:05:08 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/game.c,v 11.25 2003/06/02 16:56:53 jcb Rel $
  * game.c
  * Routines for manipulating game data structures.
  */
@@ -14,7 +14,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.8 2002/03/30 22:05:08 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.25 2003/06/02 16:56:53 jcb Rel $";
 
 #include "game.h"
 #include "sysdep.h"
@@ -110,6 +110,7 @@ static GameOptionEntry default_optiontable[] = {
   /* a real option has a name. This is just a filler. */
   { GOUnknown, "" , GOTBool, 1000000, 0, { 0 }, "", 0 }, 
   { GOTimeout, "Timeout" , GOTNat, 0, 0, { 15 }, "time limit for claims",0 },
+  { GOTimeoutGrace, "TimeoutGrace", GOTNat, 0, 0, { 5 }, "grace period when clients handle timeouts" , 0},
   { GOScoreLimit, "ScoreLimit", GOTNat, 0, 0, { 1000 }, "limit on hand score", 0 },
   { GONoLimit, "NoLimit", GOTBool, 0, 0, { 0 }, "no-limit game", 0 },
   { GOMahJongScore, "MahJongScore", GOTScore, 0, 0, { 20 }, "base score for going out", 0 },
@@ -126,6 +127,9 @@ static GameOptionEntry default_optiontable[] = {
   { GOConcealedAlmost, "ConcealedAlmost", GOTScore, 0, 0, { 0 }, "score for almost concealed hand", 0 },
   { GOLosersPurity, "LosersPurity", GOTBool, 0, 0, { 0 }, "losing hands score doubles for pure, concealed etc.", 0 },
   { GOKongHas3Types, "KongHas3Types", GOTBool, 1034, 0, { 0 }, "claimed kongs count as concealed for doubling", 0 },
+  { GOEastDoubles, "EastDoubles", GOTBool, 0, 0, { 1 }, "east pays and receives double", 0 },
+  { GOLosersSettle, "LosersSettle", GOTBool, 0, 0, { 1 }, "losers pay each other", 0 },
+  { GODiscDoubles, "DiscDoubles", GOTBool, 0, 0, { 0 }, "the discarder pays double", 0 },
   { GOEnd, "End" , GOTBool, 0, 0, { 1 }, "end of option list", 0 }
 };
 
@@ -332,8 +336,36 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       CMsgPlayerMsg *m = (CMsgPlayerMsg *)cm;
       /* do we already know this player? */
       setups;
-      if ( p ) set_player_name(p,m->name);
+      if ( p ) {
+	/* after protocol version 1034, a player message with
+	   an empty name is defined to mean "delete this player".
+	   Note that we have to refer to the global protocol_version,
+	   since the game protversion field will not be set at
+	   the only plausible times to get this command */
+	if ( (m->name && m->name[0]) || protocol_version <= 1034 ) {
+	  set_player_name(p,m->name);
+	} else {
+	  /* player message with empty name means delete it */
+	  /* if this happens at any time other than during initial
+	     connection, something's wrong. We do not allow the
+	     deletion of a player from a game. (Maybe we should,
+	     to allow substitution, but not yet!)
+	     At pversion 1038, this was changed to allow
+	     deletion until the game actually starts, rather than
+	     only until the game is set up.
+	  */
+	  if ( game_has_started(g) ) {
+	    warn("Can't delete a player after game has started");
+	    return -1;
+	  }
+	  initialize_player(p);
+	}
+      }
       else {
+	if ( protocol_version > 1034 && ! ( m->name && m->name[0] ) ) {
+	  warn("trying to delete non-existent player");
+	  return -1;
+	}
 	/* find first free player */
 	p = game_id_to_player(g,0);
 	if ( ! p ) {
@@ -381,6 +413,8 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       g->protversion = m->protversion;
       g->manager = m->manager;
       game_set_options_from_defaults(g);
+      /* and games always start inactive */
+      g->active = 0;
     }
     return affected_id;
   case CMsgInfoTiles:
@@ -472,13 +506,17 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       }
       /* clear game flags */
       g->flags = 0;
+      g->konging = NotKonging;
       g->wall.live_used = 0;
       g->wall.size = game_get_option_value(g,GOFlowers,NULL) ? 144 : 136;
       g->wall.dead_end = g->wall.size;
       g->wall.live_end = g->wall.dead_end - 
 	(game_get_option_value(g,GODeadWall,NULL) ? (game_get_option_value(g,GODeadWall16,NULL) ? 16 : 14) : 0) ;
       g->serial = 0; /* must be initialized */
-      for ( i = 0 ; i < MaxTile ; i++ ) g->exposed_tile_count[i] = 0;
+      for ( i = 0 ; i < MaxTile ; i++ ) {
+	g->exposed_tile_count[i] = 0;
+	g->discarded_tile_count[i] = 0;
+      }
     }
     return affected_id;
   case CMsgPlayerDraws:
@@ -726,6 +764,12 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  g->cmsg_err = "Can't discard now";
 	  return -1;
 	}
+	/* give a more helpful error message when the player
+	   is waiting for a tile */
+	if ( g->needs ) {
+	  g->cmsg_err = "Need to draw a tile before discarding";
+	  return -1;
+	}
 	/* players aren't allowed to discard if the game is paused */
 	if ( g->paused ) {
 	  g->cmsg_err = "Can't discard until everybody's ready";
@@ -776,7 +820,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       g->serial = m->discard;
       for ( i = 0 ; i < NUM_SEATS ; i++ )
 	g->claims[i] = UnknownClaim;
-      g->pending = 0;
+      g->chowpending = 0;
       game_clearflag(g,GFKong);
       game_clearflag(g,GFKongUponKong);
       game_clearflag(g,GFDangerousDiscard);
@@ -786,6 +830,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	if ( nodiscard ) { psetflag(p,OriginalCall); }
       }
       g->exposed_tile_count[m->tile]++;
+      g->discarded_tile_count[m->tile]++;
       return affected_id;
     }
   case CMsgPlayerDoesntClaim:
@@ -1020,7 +1065,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	/* Note that normally we assume the legality of
 	   the pair was previously checked at claim time */
 	if ( ! ( g->state == MahJonging
-		 && g->pending
+		 && g->mjpending
 		 && s == g->player ) ) {
 	  g->cmsg_err = "Nothing to pair";
 	  return -1;
@@ -1046,7 +1091,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  copy_player(p,&cp); /* restore status quo */
 	  return -1;
 	}
-	g->pending = 0;
+	g->mjpending = 0;
 	if ( chk ) mark_dangerous_discards(g);
       } else {
 	g->cmsg_err = "Nothing to pair";
@@ -1070,7 +1115,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	/* Note that normally we assume the legality of
 	   the set was previously checked at claim time */
 	if ( ! ( g->state == MahJonging
-		 && g->pending
+		 && g->mjpending
 		 && s == g->player ) ) {
 	  g->cmsg_err = "Nothing to claim";
 	  return -1;
@@ -1119,7 +1164,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  copy_player(p,&cp); /* restore status quo */
 	  return -1;
 	}
-	g->pending = 0;
+	g->mjpending = 0;
       } else {
 	g->cmsg_err = "Can't declare special set now";
 	return -1;
@@ -1138,7 +1183,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	   the pung was previously checked at claim time */
 	if ( ! (g->state == Discarded
 		|| (g->state == MahJonging
-		    && g->pending
+		    && g->mjpending
 		    && g->player == s) ) ) {
 	  g->cmsg_err = "Nothing to pung";
 	  return -1;
@@ -1179,7 +1224,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  copy_player(p,&cp); /* restore status quo */
 	  return -1;
 	}
-	g->pending = 0;
+	g->mjpending = 0;
       } else {
 	g->cmsg_err = "Nothing to chow";
 	return -1;
@@ -1233,7 +1278,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	   the chow was previously checked at claim time */
 	if ( ! (g->state == Discarded
 		|| (g->state == MahJonging
-		    && g->pending
+		    && g->mjpending
 		    && g->player == s) ) ) {
 	  g->cmsg_err = "Nothing to chow";
 	  return -1;
@@ -1248,9 +1293,33 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	 to a successful claimant to tell it to specify the position.
 	 There is therefore nothing to do here,
 	 except set the pending flag.
+	 Oops! Actually, if we're mah-jonging, we have to check that
+	 the player can actually go out by chowing the tile.
       */
       if ( m->cpos == AnyPos ) {
-	g->pending = 1;
+	if ( chk && g->state == MahJonging ) {
+	/* checking is more complex: not only must
+	   the player be able make the chow in some position, but it
+	   must then be able to complete the hand */
+	  ChowPosition pos;
+	  int ok = 0;
+	  for ( pos = Lower; pos <= Upper; pos++ ) {
+	    Player cp;
+	    copy_player(&cp,p); 
+	    if ( ! player_chows(&cp,m->tile,pos) ) {
+	      continue;
+	    }
+	    if ( player_can_mah_jong(&cp,HiddenTile,mjspecflags) ) {
+	      ok = 1;
+	      break;
+	    }
+	  }
+	  if ( ! ok ) { 
+	    g->cmsg_err = "Can't go out by chowing";
+	    return -1;
+	  }
+	}
+	g->chowpending = 1;
 	return affected_id;
       }
 
@@ -1300,7 +1369,8 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	  copy_player(p,&cp); /* restore status quo */
 	  return -1;
 	}
-	g->pending = 0;
+	g->chowpending = 0;
+	g->mjpending = 0;
       } else {
 	g->cmsg_err = "Nothing to chow";
 	return -1;
@@ -1345,7 +1415,8 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	/* g->whence is unchanged */
 	/* g->player is unchanged */
 	/* g->tile is unchanged */
-	g->pending = 0;
+	g->mjpending = 0;
+	g->chowpending = 0;
 	g->state = MahJonging;
       } else {
 	/* We should be currently in the Discarded state */
@@ -1359,7 +1430,8 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	g->whence = FromDiscard;
 	g->player = s;
 	/* g->tile is unchanged */
-	g->pending = 1;
+	g->mjpending = 1;
+	g->chowpending = 0;
 	g->state = MahJonging;
        }
       return affected_id;
@@ -1643,7 +1715,9 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       g->supplier = g->player;
       g->player = s;
       g->tile = m->tile;
-      g->pending = 1;
+      g->mjpending = 1;
+      g->chowpending = 0;
+      g->konging = NotKonging;
       game_clearflag(g,GFKong);
       game_clearflag(g,GFKongUponKong);
       return affected_id;
@@ -1676,7 +1750,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
 	g->cmsg_err = "Couldn't show those tiles";
 	return -1;
       }
-
+      /* the player_shows_tiles fn already set the HandDeclared flag */
       return affected_id;
     }
   case CMsgSwapTile:
@@ -1727,7 +1801,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       CMsgGameOptionMsg *m = (CMsgGameOptionMsg *)cm;
 
       setups;
-      if ( chk && g->manager && g->manager != m->id ) {
+      if ( chk && (g->manager != 0) && g->manager != m->id ) {
 	g->cmsg_err = "Not authorized to set game options";
 	return -1;
       }
@@ -1741,7 +1815,7 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       CMsgChangeManagerMsg *m = (CMsgChangeManagerMsg *)cm;
 
       setups;
-      if ( chk && g->manager && g->manager != m->id ) {
+      if ( chk && (g->manager != 0) && g->manager != m->id ) {
 	g->cmsg_err = "Not authorized to change the manager";
 	return -1;
       }
@@ -1787,6 +1861,20 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
   return -2;
 }
 
+/* say whether a game has started or not */
+int game_has_started(Game *g) {
+  if ( g == NULL ) return 0;
+  if ( g->round == UnknownWind ) return 0;
+  if ( g->round != EastWind ) return 1;
+  /* this next one is tricky: if there's no player in east,
+     we've already deleted it, so it must be OK to delete others */
+  if ( g->players[east]->id == 0 ) return 0;
+  if ( g->players[east]->id != g->firsteast ) return 1;
+  if ( g->hands_as_east > 0 ) return 1;
+  if ( g->state != HandComplete ) return 1;
+  return 0;
+}
+
 /* this internal function sets the danger signal flags for a player.
    It does NOT deal with the DangerEnd flag.
    The game argument is currently unused.
@@ -1838,18 +1926,12 @@ static void set_danger_flags(Game *g UNUSED, PlayerP p) {
       allcircle = allcirclehonour = 0;
       switch ( t->type ) {
       case Chow:
-	if ( value_of(t->tile) != 2 ) allgreen = 0;
+	if ( !is_green(t->tile) 
+	  || !is_green(t->tile+1)
+	  || !is_green(t->tile+2) ) allgreen = 0;
 	break;
       default:
-	switch ( value_of(t->tile) ) {
-	case 1:
-	case 5:
-	case 7:
-	case 9:
-	  allgreen = 0;
-	default:
-	  ;
-	}
+	if ( !is_green(t->tile) ) allgreen = 0;
       }
       break;
     case CharacterSuit:
@@ -1864,7 +1946,7 @@ static void set_danger_flags(Game *g UNUSED, PlayerP p) {
       break;
     case DragonSuit:
       allbamboo = allcharacter = allcircle = 0;
-      if ( value_of(t->tile) != GreenDragon ) allgreen = 0;
+      if ( !is_green(t->tile) ) allgreen = 0;
       break;
     case WindSuit:
       allbamboo = allcharacter = allcircle = 0;
@@ -1945,7 +2027,7 @@ static void mark_dangerous_discards(Game *g) {
 
   /* End danger is special */
   if ( g->wall.live_end - g->wall.live_used <= 4 
-       && g->exposed_tile_count[t] == 0 ) danger |= DangerEnd;
+       && g->discarded_tile_count[t] <= 1 ) danger |= DangerEnd;
 
   /* if we've determined that the discard was dangerous,
      we now have to see whether the supplier had no choice */
@@ -1975,7 +2057,7 @@ static void mark_dangerous_discards(Game *g) {
 	    || ( pdflag(pp,i,DangerTerminal) && is_terminal(t) )
 	    || ( pdflag(pp,i,DangerGreen) && is_green(t) )
 	    || ( g->wall.live_end - g->wall.live_used <= 4 
-		 && g->exposed_tile_count[t] == 0 ) 
+		 && g->discarded_tile_count[t] <= 1 ) 
 	    ) dangerous = 1;
       }
       if ( ! dangerous ) {
