@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.5 2002/01/02 15:01:10 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.6 2003/03/30 21:33:27 jcb Rel $
  * greedy.c
  * This is a computer player. Currently offensive only.
  * Options not documented.
@@ -15,7 +15,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.5 2002/01/02 15:01:10 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.6 2003/03/30 21:33:27 jcb Rel $";
 
 static int debugeval = 0;
 #include <stdlib.h>
@@ -453,7 +453,7 @@ static void do_something(void) {
     /* get the list of possible decls */
     tsp = client_find_sets(our_player,
 			   (the_game->player == our_seat
-			    && the_game->pending)
+			    && the_game->mjpending)
 			   ? the_game->tile : HiddenTile,
 			   the_game->player == our_seat,
 			   (PlayerP *)0,mjspecflags);
