@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/gui.h,v 11.12 2002/01/05 23:31:26 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/gui.h,v 11.29 2003/04/16 22:28:58 jcb Rel $
  * gui.h
  * type defns and forward declarations for the gui module.
  */
@@ -170,6 +170,21 @@ typedef enum {
  DialogsBelow = 2,
  DialogsPopup = 3 } DialogPosition;
 
+/* The rcfile is used to store assorted persistent data. We often
+   need to read or update only selected parts of it. This set of
+   flag bits identifies the groups that can be independently 
+   read or updated. */
+typedef enum {
+  XmjrcNone = 0x00,
+  XmjrcDisplay = 0x01, /* preferences regarding the display */
+  XmjrcPlayer = 0x02, /* player option settings */
+  XmjrcGame = 0x04, /* game option preferences */
+  XmjrcMisc = 0x08, /* odds and sods */
+  XmjrcOpen = 0x10, /* sticky fields in the connection dialogs */
+  XmjrcPlaying = 0x20, /* playing preferences */
+  XmjrcAll = 0xFFFFFFFF /* everything */
+} XmjrcGroup;
+
 /* extra data in the game */
 typedef struct {
   int orig_live_end; /* value of wall.live_end at start of hand */
@@ -216,14 +231,13 @@ void nag_popup(void);
 void option_reset_callback(GtkWidget *w, gpointer data);
 void option_updater_callback(GtkWidget *w, gpointer data);
 void open_connection(GtkWidget *w UNUSED, gpointer data);
-void open_dialog_init(char *ht, char *pt, char *ft, char *idt, char *nt, int usehost);
+void open_dialog_init(char *idt, char *nt);
 void open_dialog_popup(GtkWidget *w UNUSED, gpointer data);
+void playing_prefs_init(void);
 void prefs_updater_callback(GtkWidget *w, gpointer data);
 int read_or_update_rcfile(char *rcfile,
-			  int do_display,
-			  int do_player,
-			  int do_game,
-			  int do_misc);
+  XmjrcGroup read_groups,
+  XmjrcGroup update_groups);
 void scoring_dialog_init(void);
 void scoring_dialog_popup(void);
 void set_dialog_posn(DialogPosition p);
@@ -240,6 +254,9 @@ void tilesetbox_set(TileSetBox *tb, const TileSet *ts, int ori);
 void turn_dialog_init(void);
 void turn_dialog_popup(void);
 void usage(char *pname,char *msg);
+void warningwindow_init(void);
+int log_msg_add(LogLevel l,char *warning);
+void warning_clear(void);
 /* Convenience function */
 #define send_packet(m) client_send_packet(the_game,(PMsgMsg *)m)
 
@@ -247,6 +264,8 @@ void usage(char *pname,char *msg);
 
 /* in gui.c */
 extern int debug;
+extern int server_pversion; /* protocol version that server speaks
+			       (independently of game) */
 extern PlayerP our_player;
 extern int our_id;
 extern seats our_seat; /* our seat in the game */
@@ -255,6 +274,7 @@ extern int selected_button; /* the index of the user's selected
 			     tile, or -1 if none */
 
 extern int ptimeout; /* claim timeout time in milliseconds */
+extern int local_timeouts; /* are we handling timeouts ourselves? */
 
 extern GdkPixmap **tilepixmaps[]; /* pixmaps for the tiles */
 
@@ -318,6 +338,9 @@ extern char robot_names[3][128];
 
 /* The window for messages, and the display text widget */
 extern GtkWidget *messagewindow, *messagetext;
+/* and for warnings */
+extern GtkWidget *warningwindow, *warningtext;
+extern GtkWidget *info_box; /* box to hold info windows */
 /* completed games, for nagging */
 extern int completed_games;
 extern int nag_state; /* has user been nagged to pay? */
@@ -328,13 +351,30 @@ extern GtkWidget *nag_window;
    of tiles */
 
 extern int pdispwidth;
+extern int display_size; /* and this is the preference value for pdispwidth */
 extern int square_aspect; /* force a square table */
+extern char address[]; /* server address */
+extern char name[]; /* player's name */
+extern char main_font_name[]; /* font used in all dialogs etc */
+extern char text_font_name[]; /* font used for text in windows */
+extern char fallback_text_font_name[]; /* one that worked */
+extern char table_colour_name[]; /* colour of the table background */
 extern int animate; /* do fancy animation */
+extern int nopopups; /* suppress automatic popup of message/scoring windows */
 extern int tiletips; /* show tiletips constantly */
 extern int showwall; /* show the wall or not */
+extern int info_windows_in_main; /* should the message and game info windows
+				    be included in the main window, or
+				    be separate popups ? */
 extern int iconify_dialogs_with_main; /* as it says ... */
 extern char *tileset, *tileset_path; /* tile sets */
 extern int pref_showwall; /* preferred value of showwall */
+
+/* playing options */
+extern int playing_auto_declare_specials;
+extern int playing_auto_declare_losing;
+extern int playing_auto_declare_winning;
+
 /* the player display areas */
 extern PlayerDisp pdisps[NUM_SEATS];
 
@@ -372,7 +412,7 @@ extern GtkWidget *game_prefs_dialog;
 extern GtkWidget *game_option_panel;
 extern GtkWidget *game_prefs_panel;
 extern GameOptionTable prefs_table;
-
+extern GtkWidget *playing_prefs_dialog;
 /* static for each module */
 #include "version.h"
 
