@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/player.c,v 11.3 2002/03/30 22:49:22 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/player.c,v 11.4.1.1 2004/05/09 21:49:56 jcb Exp $
  * player.c
  * Implements functions on players.
  */
@@ -14,7 +14,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/player.c,v 11.3 2002/03/30 22:49:22 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/player.c,v 11.4.1.1 2004/05/09 21:49:56 jcb Exp $";
 
 #define PLAYER_C /* so that PlayerP is not const */
 #include "player.h"
@@ -754,15 +754,15 @@ int player_can_thirteen_wonders(PlayerP p, Tile d) {
   Player pcopy; /* we will manipulate this copy of the player */
   PlayerP pcp = &pcopy;
   int answer = 0;
-  int i, havetwo, n;
+  int i, n;
 
   if ( d != HiddenTile ) {
     copy_player(pcp,p);
     answer = (player_draws_tile(pcp,d)
 	      && player_can_thirteen_wonders(pcp,HiddenTile));
   } else {
+    int havetwo = 0;
     answer = 1;
-    havetwo = 0;
     for ( i=0; i < 13; i++ ) {
       n = player_count_tile(p,thirteen_wonders[i]);
       if ( n > 2 ) { answer = 0; break; }
@@ -771,6 +771,7 @@ int player_can_thirteen_wonders(PlayerP p, Tile d) {
 	else { havetwo = 1; }
       } else if ( n == 0 ) { answer = 0; break; }
     }
+    if ( ! havetwo ) answer = 0;
   }
   return answer;
 }
