@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/protocol.h,v 11.6 2002/03/30 18:43:29 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/protocol.h,v 11.15 2003/10/06 19:58:19 jcb Rel $
  * protocol.h
  * defines the messages passed between players and controller
  */
@@ -46,6 +46,7 @@ typedef enum {
   /* make-enums sub { $_[0] =~ s/^GO//; } */
   GOUnknown=0,
   GOTimeout, /* the timeout in seconds for claims. Default: 15 */
+  GOTimeoutGrace, /* grace period when clients handle timeouts locally */
   GOScoreLimit, /* limit for score. Default 1000 */
   GONoLimit, /* true if no limit on scores. Then ScoreLimit is
 	       notional value of limit hand */
@@ -71,6 +72,9 @@ typedef enum {
   GOLosersPurity, /* do losers score the doubles for pure and almost 
 		       concealed hand? */
   GOKongHas3Types, /* do we use Millington's 3-way kong distinction? */
+  GOEastDoubles, /* does East pay and receive double (as in Chinese Classical)? */
+  GOLosersSettle, /* do losers pay among themselves? */
+  GODiscDoubles, /* does the discarder pay a double score? */
   GOEnd } GameOption;
 #define GONumOptions (GOEnd+1)
 
@@ -128,7 +132,18 @@ typedef struct _GameOptionEntry {
    a version of the protocol plus options etc.
 */
 
-#define PROTOCOL_VERSION 1034
+/* This define gives the protocol version implemented in this file */
+#define PROTOCOL_VERSION 1060
+
+/* This global variable, found in protocol.c, is used by many functions
+   to determine the protocol version currently being used.
+   It would be better if this were an argument to every function that
+   used it, or if every function were passed a Game structure, but this
+   would be too inconvenient. So this global variable is used instead.
+   If we ever program with different games in the same process, care will
+   have to be taken to manage this.
+*/
+extern int protocol_version;
 
 /* These structures are the programmer's view of the messages.
    The protocol is actually defined by the wire protocol, not by
@@ -198,6 +213,7 @@ typedef enum {
 */
 typedef enum {
   PMsgSaveState = 1,
+  PMsgLoadState = 2,
   PMsgConnect = 10,
   PMsgDeclareSpecial = 33,
   PMsgRequestPause = 36,
@@ -271,6 +287,16 @@ typedef struct _CMsgStateSavedMsg {
 		     system dependent form */
 } CMsgStateSavedMsg;
 
+/* This message requests the server to load the game state
+   from a saved file. There is no reply on success; an error
+   is returned on failure. */
+/* This message was introduced at pversion 1038. */
+typedef struct _PMsgLoadStateMsg {
+  PlayerMsgType type; /* PMsgSaveState */
+  char *filename; /* name of file to load; subject to interpretation
+		     and overriding by server. */
+} PMsgLoadStateMsg;
+
 /* This message is purely informational, and is mainly useful
    for debugging or for very dumb clients. It may be sent to
    a player whenever its hand changes, and contains a human
@@ -283,7 +309,7 @@ typedef struct _CMsgInfoTilesMsg {
   char *tileinfo;
 } CMsgInfoTilesMsg;
 
-/* Message sent by player after connecting to controller */
+/* Message sent by player after connecting to controller. */
 
 typedef struct _PMsgConnectMsg {
   PlayerMsgType type; /* PMsgConnect */
@@ -306,6 +332,9 @@ typedef struct _CMsgConnectReplyMsg {
    existing player; and each existing player receives a message for the
    new player.
    This message is also sent if a player changes its name.
+   After protocol version 1034, if the name is empty, this means
+   "delete the player". (Empty names were previously permitted,
+   but this was always a mistake.)
 */
 typedef struct _CMsgPlayerMsg {
   ControllerMsgType type; /* CMsgPlayer */
@@ -1000,6 +1029,7 @@ CMsgMsg *decode_cmsg(char *arg);
    null).
    Returns 1 on success, 0 on internal errors (e.g. missing arguments),
    and -1 if dest is not long enough. dest will always be null-terminated.
+   If src is null, expands to the empty string.
 */
 
 int basic_expand_protocol_text(char *dest,const char *src,int n);
