@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/scoring.c,v 11.5 2002/03/30 22:49:50 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/scoring.c,v 11.8 2003/04/13 18:49:51 jcb Rel $
  * scoring.c 
  * This file contains the routines to handle the scoring
  * of hands.
@@ -15,7 +15,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/scoring.c,v 11.5 2002/03/30 22:49:50 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/scoring.c,v 11.8 2003/04/13 18:49:51 jcb Rel $";
 
 #include "tiles.h"
 #include "player.h"
@@ -167,7 +167,7 @@ Score score_of_hand(Game *g, seats s) {
       doubles += dbls; sprintf(tbuf,"%-12s  %1d dbl (%s)\n",ltext,dbls,rtext); \
       strcat(dblsbuf,tbuf); \
     } \
-    if ( clims ) { \
+    if ( clims && !(nolimit && (pts || dbls)) ) { \
       if ( clims > centilims ) centilims = clims; \
       if ( clims == 100 ) { \
         sprintf(tbuf,"%-10s %4d (%s)\n","Limit Hand",limit,rtext); \
@@ -276,18 +276,12 @@ Score score_of_hand(Game *g, seats s) {
       switch ( t->type ) {
       case Chow:
       case ClosedChow:
-	if ( value_of(t->tile) != 2 ) allgreen = 0;
+	if ( !is_green(t->tile) 
+	  || !is_green(t->tile+1)
+	  || !is_green(t->tile+2) ) allgreen = 0;
 	break;
       default:
-	switch ( value_of(t->tile) ) {
-	case 1:
-	case 5:
-	case 7:
-	case 9:
-	  allgreen = 0;
-	default:
-	  ;
-	}
+	if ( !is_green(t->tile) ) allgreen = 0;
       }
       break;
     case CharacterSuit:
@@ -302,7 +296,7 @@ Score score_of_hand(Game *g, seats s) {
       break;
     case DragonSuit:
       allbamboo = allcharacter = allcircle = 0;
-      if ( value_of(t->tile) != GreenDragon ) allgreen = 0;
+      if ( !is_green(t->tile) ) allgreen = 0;
       break;
     case WindSuit:
       allbamboo = allcharacter = allcircle = 0;
@@ -319,6 +313,9 @@ Score score_of_hand(Game *g, seats s) {
   /* now go through the concealed tiles, in case we're playing some
      weird game where losers are allowed to score for pure and concealed
      hands. Seems a pity to duplicate all this code, but there we are */
+  /* There's another reason to go through the concealed tiles: the
+     existence of special hands, where the concealed tiles are
+     a declared set */
   for ( i = 0; i < p->num_concealed; i++ ) {
     Tile ti = p->concealed[i];
 
@@ -326,20 +323,25 @@ Score score_of_hand(Game *g, seats s) {
     case BambooSuit:
       allcharacter = allcharacterhonour = 0;
       allcircle = allcirclehonour = 0;
+      if ( !is_green(ti) ) allgreen = 0;
       break;
     case CharacterSuit:
       allbamboo = allbamboohonour = 0;
       allcircle = allcirclehonour = 0;
+      allgreen = 0;
       break;
     case CircleSuit:
       allbamboo = allbamboohonour = 0;
       allcharacter = allcharacterhonour = 0;
+      allgreen = 0;
       break;
     case DragonSuit:
       allbamboo = allcharacter = allcircle = 0;
+      if ( !is_green(ti) ) allgreen = 0;
       break;
     case WindSuit:
       allbamboo = allcharacter = allcircle = 0;
+      allgreen = 0;
       break;
     default:
       warn("Strange suit seen in hand");
@@ -419,7 +421,6 @@ Score score_of_hand(Game *g, seats s) {
 #define dbl(n,ex) doscore("",n DBLS,ex)
 
   /* Now start looking for doubles from the tilesets */
-  doubles = 0;
   for ( i = 0; i < MAX_TILESETS; i++ ) {
     TileSetP t = (TileSetP) &p->tilesets[i];
 
@@ -736,7 +737,12 @@ Score score_of_hand(Game *g, seats s) {
   /* calculate the doubles */
   sprintf(tbuf,"%-12s %2d\n","total dbls",doubles);
   strcat(buf,tbuf);
-  while ( doubles-- > 0 ) tot.value *= 2;
+  /* to avoid arithmetic overflow when people set crazy options,
+     we'll put a hard limit of 1E8 in */
+  while ( doubles-- > 0 ) {
+    tot.value *= 2;
+    if ( tot.value > 100000000 ) tot.value = 100000000 ;
+  }
   if ( ! nolimit && tot.value > limit ) {
     sprintf(tbuf,"%-9s %5d (over limit)\n","score:",tot.value);
     strcat(buf,tbuf);
