@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.4 2002/01/02 14:58:08 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.17 2004/04/16 06:33:06 jcb Exp $
  * sysdep.c
  * By intention, this file contains all functions that might need
  * fiddling with to cope with different variants of Unix.
@@ -21,7 +21,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.4 2002/01/02 14:58:08 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.17 2004/04/16 06:33:06 jcb Exp $";
 
 #include "sysdep.h"
 #include <string.h>
@@ -49,13 +49,26 @@ static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.4 2002/
 #endif
 
 /* warn: = fprintf(stderr,...), with automatic new line */
-int warn(char *format,...) {
-  va_list args;
-  static char buf[256];
+/* log_msg is a generalized version with a syslog like warning
+   level. Currently only two levels are defined by the
+   LogLevel enum, LogWarning and LogInfo. */
+
+
+int (*log_msg_hook)(LogLevel l,char *);
+
+static char *log_prefixes[] = { "" , "-I- ", "+W+ " };
+
+/* internal function */
+static int vlog_msg(LogLevel l, char *format,va_list args) {
+  char buf[1024];
   int ret;
   int n;
-  va_start(args,format);
-  strncpy(buf,format,250);
+
+  strcpy(buf,log_prefixes[l]);
+  n = strlen(log_prefixes[l]);
+  /* reserve 1 char for extra '\n' */
+  ret = vsnprintf(buf+n,sizeof(buf)-n-1,format,args);
+  buf[1023] = 0;
   n = strlen(buf);
   /* do quick check for terminators */
 #ifdef WIN32
@@ -72,11 +85,31 @@ int warn(char *format,...) {
 #else
   if ( buf[n-1] != '\n' ) strcat(buf,"\n");
 #endif
-  ret = vfprintf(stderr,buf,args);
-  va_end(args);
+  /* call the hook */
+  if ( log_msg_hook == NULL || log_msg_hook(l,buf) == 0 ) {
+    fprintf(stderr,"%s",buf);
+  }
   return ret;
 }
 
+int log_msg(LogLevel l,char *format,...) {
+  va_list args;
+  va_start(args,format);
+  return vlog_msg(l,format,args);
+}
+
+int warn(char *format,...) {
+  va_list args;
+  va_start(args,format);
+  return vlog_msg(LogWarning,format,args);
+}
+
+int info(char *format,...) {
+  va_list args;
+  va_start(args,format);
+  return vlog_msg(LogInfo,format,args);
+}
+
 /* ignore the SIGPIPE signal. Return 0 on success, -1 on error */
 int ignore_sigpipe(void) {
 #ifdef WIN32
@@ -377,6 +410,7 @@ int close_socket(SOCKET s) {
 
 /* read a line, up to lf/crlf terminator, from the socket,
    and return it.
+   Internal backslashes are removed, unless doubled.
    If the line ends with an unescaped backslash, replace by newline,
    and append the next line.
    If there is an error or end of file before seeing a terminator,
@@ -386,13 +420,22 @@ int close_socket(SOCKET s) {
    the second arg says whether the first is int or SOCKET.
 */
 static char *_get_line(int fd,int is_socket) {
-  int length;
+  int length,i,j;
   int offset=0;
-  static char buffer[1024];
+  static char *buffer = NULL;
+  static size_t buffer_size = 0;
 
   while ( 1 ) {
-    while ( ( (length
-      = (is_socket ?
+    while ( 1 ) {
+      while (offset+1 >= (int)buffer_size) { /* +1 to keep some space for '\0' */
+	buffer_size = (buffer_size == 0) ? 1024 : (2 * buffer_size);
+	buffer = realloc(buffer, buffer_size);
+	if (!buffer) {
+	  perror("get_line");
+	  exit(-1);
+	}
+      }
+      length = (is_socket ?
 	(skt_reader ? skt_reader((void *)fd,buffer+offset,1)
 		  : 
 #ifdef WIN32
@@ -400,9 +443,11 @@ static char *_get_line(int fd,int is_socket) {
 #else
 		  read(fd,buffer+offset,1)
 #endif
-	 ) : read(fd,buffer+offset,1))) > 0 )
-	    && buffer[offset]!='\n')
+	 ) : read(fd,buffer+offset,1));
+      if ((length <= 0) || (buffer[offset] == '\n'))
+	break;
       offset += length;
+    }
     if ( offset == 0 ) return NULL;
     if ( length <= 0 ) {
       fprintf(stderr,"get_line read partial line");
@@ -422,7 +467,7 @@ static char *_get_line(int fd,int is_socket) {
       if ( buffer[offset-2] == '\\' ) {
 	/* we have to decide whether this is an escaped backslash */
 	int j = 1,k=offset-3;
-	while ( k >= 0 && buffer[k] == '\\' ) j++;
+	while ( k >= 0 && buffer[k--] == '\\' ) j++;
 	if ( j & 1 ) {
 	  /* odd number, not escaped */
 	  buffer[offset-2] = '\n' ;
@@ -432,10 +477,21 @@ static char *_get_line(int fd,int is_socket) {
       } else break; /* return this line */
     } /* no newline, keep going */
   }
+  /* now remove internal backslashes */
+  for ( i = j = 0; 1; i++, j++ ) {
+    if ( buffer[j] == '\\' ) {
+      j++;
+    }
+    buffer[i] = buffer[j];
+    if ( !buffer[i] ) break;
+  }
   return buffer;
 }
 
 char *get_line(SOCKET fd) {
+  /* are we actually being asked to read from stdout? Then read
+     from stdin */
+  if ( fd == STDOUT_FILENO ) return _get_line(STDIN_FILENO,0);
   return _get_line((int)fd,1);
 }
 
@@ -443,54 +499,69 @@ char *fd_get_line(int fd) {
   return _get_line(fd,0);
 }
 
+static int _put_line_aux(int fd, char* line, int length, int is_socket) {
+  int n;
+  if ( is_socket ) {
+    n = skt_writer ? skt_writer((void *)fd,line,length)
+      :
+#ifdef WIN32
+      send(fd,line,length,0)
+#else
+      write(fd,line,length)
+#endif
+      ;
+  } else {
+    n = write(fd,line,length);
+  }
+  if ( n < length ) {
+    perror("put_line: write failed");
+    return -1;
+  }
+  return n;
+}
+
 /* write a line (which is assumed to have any terminator included)
    to the given socket. Return -1 on error, length of line on success.
    Replace internal newlines by backslash CRLF and write as lines.
+   Double all internal backslashes.
    Third arg says whether first arg is really an int, or a SOCKET.
    This is the internal function implemented the next two.
 */
 static int _put_line(int fd,char *line,int is_socket) {
   int length,n;
-  int nn = 0; /* return value */
-  char *newline;
-  char buffer[1024];
+  static char *buffer = NULL;
+  static size_t buf_size = 0;
+  int i,j;
+
+  /* the backslash/newline escaping can at worst treble the size
+     of the input */
+  length = strlen(line) + 1; /* include null */
+  while ( (int)buf_size <= 3*length ) {
+    buf_size = buf_size ? 2*buf_size : 1024;
+    buffer = realloc(buffer,buf_size);
+  }
 
   /* FIXME: should retry if only write partial line */
-  while ( (length = strlen(line)) > 0 ) {
-    buffer[0] = '\0';
-    newline = strchr(line,'\n');
-    if ( newline && newline[1] != '\0' ) {
-      strncpy(buffer,line,length=(newline-line));
-      buffer[length] = '\0';
-      strcat(buffer,"\\\r\n");
-      length += 3;
-      line = newline+1;
+  i = j = 0;
+  while ( *line ) {
+    if ( *line == '\\' ) { buffer[i++] = '\\' ; }
+    if ( *line == '\n' && *(line+1) ) {
+      buffer[i++] = '\\' ;
+      buffer[i++] = '\r' ;
+      buffer[i++] = *(line++) ;
+      n = _put_line_aux(fd,buffer,i,is_socket);
+      if ( n < 0 ) { return -1 ; }
+      i = 0;
     } else {
-      strcpy(buffer,line);
-      line += length;
+      buffer[i++] = *(line++);
     }
-    if ( is_socket ) {
-      n = skt_writer ? skt_writer((void *)fd,buffer,length)
-	:
-#ifdef WIN32
-	send(fd,buffer,length,0)
-#else
-	write(fd,buffer,length)
-#endif
-	;
-    } else {
-      n = write(fd,buffer,length);
-    }
-    if ( n < length ) {
-      perror("put_line: write failed");
-      return -1;
-    }
-    nn += n;
   }
-  return nn;
+  return _put_line_aux(fd,buffer,i,is_socket);
 }
 
 int put_line(SOCKET fd,char *line) {
+  /* are we asked to write to stdout? */
+  if ( fd == STDOUT_FILENO ) return _put_line(STDOUT_FILENO,line,0);
   return _put_line((int)fd,line,1);
 }
 
@@ -668,14 +739,12 @@ char *get_homedir(void) {
   if ( (p = getenv("HOMEDRIVE")) ) strcpy(buf,p);
   if ( (p = getenv("HOMEPATH")) ) strcat(buf,p);
   if ( buf[0] ) return buf;
-  warn("No home directory, using root");
-  return "C:/";
+  return NULL;
 # else
   struct passwd *pw;
   if ( (p = getenv("HOME")) ) return p;
   if ( (pw = getpwuid(getuid())) ) return pw->pw_dir;
-  warn("No home directory found; using /tmp");
-  return "/tmp";
+  return NULL;
 # endif
 }
 
@@ -724,4 +793,5 @@ unsigned int sleep(unsigned int t) { Sleep(1000*t); return 0; }
 /* I don't think we really need submillisecond resolution ... */
 unsigned int usleep(unsigned int t) { Sleep(t/1000); return 0; }
 
+
 #endif /* WIN32 */
